#!/usr/bin/env node

// third party lib definitions
const yargs = require("yargs");
const pkg = require('../package.json');
const chalk = require("chalk");

// local variables
const package = require('../package.json');
const helper = require('./utils/helper.js');
const tool = require('./utils/tool.js');
const flow = require('./utils/flow.js');

async function run() {

    try {

        // check if the yaba cli has newer version
        await tool.checkUpdate();

        // yaba commands
        const options = yargs
            .usage("Usage: yaba -o <owner> -r <repository> -t <tag> -n <release-name> -b <body> -d <draft> -c")
            .option("o", { alias: "owner", describe: "The repository owner.", type: "string" })
            .option("r", { alias: "repo", describe: "The repository name.", type: "string" })
            .option("t", { alias: "tag", describe: "The name of the tag.", type: "string" })
            .option("n", { alias: "release-name", describe: "The name of the release.", type: "string" })
            .option("b", { alias: "body", describe: "Text describing the contents of the tag. If not provided, the default changelog will be generated with the usage of the difference of master and latest release.", type: "string" })
            .option("d", { alias: "draft", describe: "`true` or only using `-d` makes the release a draft.", type: "boolean" })
            .option("c", { alias: "changelog", describe: "Shows only changelog without creating the release.", type: "boolean" })
            .epilog(`Copyleft ${new Date().getFullYear()} ${package.author} - ${package.githubProfile}`)
            .alias('h', 'help')
            .help('help')
            .alias('v', 'version')
            .version(pkg.version)
            .argv;


        // check required ENV variables
        flow.checkRequiredEnvVariables();

        // check if the current directory is git repo
        if (options.repo == undefined && !helper.isGitRepo()) {
            error(`The directory '${helper.retrieveCurrentDirectory()}' is not a Git repo.`);
            return;
        }

        // check internet connection
        await flow.checkInternetConnection();

        const user = await flow.getAuthenticatedUser();
        const repoOwner = helper.retrieveOwner(options.owner, user.login);
        const releaseRepo = helper.retrieveReleaseRepo(options.repo);

        // fetch latest release
        const latestRelease = await flow.fetchLatestRelease(repoOwner, releaseRepo);

        // fetch head branch
        const headBranch = await flow.fetchHeadBranch(repoOwner, releaseRepo);

        // prepare changelog
        const changeLog = await flow.prepareChangelog(repoOwner, releaseRepo, latestRelease.tag_name, headBranch);

        // show only changelog
        if (changeLog.commits.length != 0 && options.changelog) {
            console.log('\n\n' + chalk.green.underline(`${releaseRepo} changelog for upcoming release:`) + `\n\n${helper.prepareChangeLog(options.body, changeLog)}\n`);
        }

        // create the release
        if (changeLog.commits.length != 0 && !options.changelog) {
            await flow.createRelease(repoOwner, releaseRepo, options.draft, options.releaseName, helper.prepareChangeLog(options.body, changeLog), options.tag);
        }

    } catch (error) {
        console.log(error);
    }
}

run();
